package javax.naming;

import java.io.StreamCorruptedException;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Enumeration;

public class CompositeName implements Name
{
    private transient NameImpl impl;
    private static final long serialVersionUID = 1667768148915813118L;
    
    protected CompositeName(Enumeration<String> enumeration) {
        this.impl = new NameImpl(null, enumeration);
    }
    
    public CompositeName(String s) throws InvalidNameException {
        this.impl = new NameImpl(null, s);
    }
    
    public CompositeName() {
        this.impl = new NameImpl(null);
    }
    
    @Override
    public String toString() {
        return this.impl.toString();
    }
    
    @Override
    public boolean equals(Object o) {
        return o != null && o instanceof CompositeName && this.impl.equals(((CompositeName)o).impl);
    }
    
    @Override
    public int hashCode() {
        return this.impl.hashCode();
    }
    
    @Override
    public int compareTo(Object o) {
        if (!(o instanceof CompositeName)) {
            throw new ClassCastException("Not a CompositeName");
        }
        return this.impl.compareTo(((CompositeName)o).impl);
    }
    
    @Override
    public Object clone() {
        return new CompositeName(this.getAll());
    }
    
    @Override
    public int size() {
        return this.impl.size();
    }
    
    @Override
    public boolean isEmpty() {
        return this.impl.isEmpty();
    }
    
    @Override
    public Enumeration<String> getAll() {
        return this.impl.getAll();
    }
    
    @Override
    public String get(int n) {
        return this.impl.get(n);
    }
    
    @Override
    public Name getPrefix(int n) {
        return new CompositeName(this.impl.getPrefix(n));
    }
    
    @Override
    public Name getSuffix(int n) {
        return new CompositeName(this.impl.getSuffix(n));
    }
    
    @Override
    public boolean startsWith(Name name) {
        return name instanceof CompositeName && this.impl.startsWith(name.size(), name.getAll());
    }
    
    @Override
    public boolean endsWith(Name name) {
        return name instanceof CompositeName && this.impl.endsWith(name.size(), name.getAll());
    }
    
    @Override
    public Name addAll(Name name) throws InvalidNameException {
        if ((name instanceof CompositeName)) {
            this.impl.addAll(name.getAll());
            return this;
        }
        throw new InvalidNameException("Not a composite name: " + name.toString());
    }
    
    @Override
    public Name addAll(int n, Name name) throws InvalidNameException {
        if ((name instanceof CompositeName)) {
            this.impl.addAll(n, name.getAll());
            return this;
        }
        throw new InvalidNameException("Not a composite name: " + name.toString());
    }
    
    @Override
    public Name add(String s) throws InvalidNameException {
        this.impl.add(s);
        return this;
    }
    
    @Override
    public Name add(int n, String s) throws InvalidNameException {
        this.impl.add(n, s);
        return this;
    }
    
    @Override
    public Object remove(int n) throws InvalidNameException {
        return this.impl.remove(n);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.writeInt(this.size());
        Enumeration<String> all = this.getAll();
        while (all.hasMoreElements()) {
            objectOutputStream.writeObject(all.nextElement());
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        this.impl = new NameImpl(null);
        int int1 = objectInputStream.readInt();
        try {
            while (--int1 >= 0) {
                this.add((String)objectInputStream.readObject());
            }
        }
        catch (InvalidNameException ex) {
            throw new StreamCorruptedException("Invalid name");
        }
    }
}
